@@ -7,6 +7,7 @@ use self::{
 use crate::es2015::classes::SuperFieldAccessFolder;
 use std::{collections::HashSet, mem::take};
 use swc_atoms::JsWord;
+use swc_common::SyntaxContext;
 use swc_common::{util::move_map::MoveMap, Mark, Spanned, DUMMY_SP};
 use swc_ecma_ast::*;
 use swc_ecma_transforms_base::helper;
@@ -37,6 +38,7 @@ pub fn class_properties() -> impl Fold {
     ClassProperties {
         typescript: false,
         mark: Mark::root(),
+        method_mark: Mark::root(),
     }
 }
 
@@ -46,6 +48,7 @@ pub fn typescript_class_properties() -> impl Fold {
     ClassProperties {
         typescript: true,
         mark: Mark::root(),
+        method_mark: Mark::root(),
     }
 }
 
@@ -53,6 +56,7 @@ pub fn typescript_class_properties() -> impl Fold {
 struct ClassProperties {
     typescript: bool,
     mark: Mark,
+    method_mark: Mark,
 }
 
 #[fast_path(ShouldWork)]
@@ -317,6 +321,7 @@ impl ClassProperties {
     ) -> (Vec<VarDeclarator>, ClassDecl, Vec<Stmt>) {
         // Create one mark per class
         self.mark = Mark::fresh(Mark::root());
+        self.method_mark = Mark::fresh(Mark::root());
 
         let has_super = class.super_class.is_some();
 
@@ -327,12 +332,11 @@ impl ClassProperties {
         let mut used_names = vec![];
         let mut used_key_names = vec![];
         let mut statics = HashSet::default();
+        let mut private_methods = HashSet::default();
 
         for member in class.body {
             match member {
-                ClassMember::PrivateMethod(..)
-                | ClassMember::Empty(..)
-                | ClassMember::TsIndexSignature(..) => members.push(member),
+                ClassMember::Empty(..) | ClassMember::TsIndexSignature(..) => members.push(member),
 
                 ClassMember::Method(method) => {
                     // we handle computed key here to preserve the execution order
@@ -660,6 +664,64 @@ impl ClassProperties {
 
                     constructor = Some(c);
                 }
+
+                ClassMember::PrivateMethod(method) => {
+                    private_methods.insert(method.key.id.sym.clone());
+
+                    let prop_span = method.span;
+                    let fn_name = Ident::new(
+                        method.key.id.sym.clone(),
+                        method
+                            .span
+                            .with_ctxt(SyntaxContext::empty())
+                            .apply_mark(self.method_mark),
+                    );
+                    if method.is_static {
+                        statics.insert(method.key.id.sym.clone());
+                    }
+
+                    let weak_set_var = Ident::new(
+                        format!("_{}", method.key.id.sym).into(),
+                        // We use `self.mark` for private variables.
+                        method.key.span.apply_mark(self.mark),
+                    );
+                    method.function.visit_with(
+                        &Invalid { span: DUMMY_SP } as _,
+                        &mut UsedNameCollector {
+                            used_names: &mut used_names,
+                        },
+                    );
+
+                    vars.push(VarDeclarator {
+                        span: DUMMY_SP,
+                        definite: false,
+                        name: Pat::Ident(weak_set_var.clone().into()),
+                        init: Some(Box::new(Expr::from(NewExpr {
+                            span: DUMMY_SP,
+                            callee: Box::new(Expr::Ident(quote_ident!("WeakSet"))),
+                            args: Some(Default::default()),
+                            type_args: Default::default(),
+                        }))),
+                    });
+
+                    // Add `_get.add(this);` to the constructor where `_get` is the name of the weak
+                    // set.
+                    constructor_exprs.push(Box::new(Expr::Call(CallExpr {
+                        span: prop_span,
+                        callee: weak_set_var
+                            .clone()
+                            .make_member(quote_ident!("add"))
+                            .as_callee(),
+                        args: vec![ThisExpr { span: DUMMY_SP }.as_arg()],
+                        type_args: Default::default(),
+                    })));
+
+                    extra_stmts.push(Stmt::Decl(Decl::Fn(FnDecl {
+                        ident: fn_name,
+                        function: method.function,
+                        declare: false,
+                    })))
+                }
             }
         }
 
@@ -676,6 +738,8 @@ impl ClassProperties {
 
         let members = members.fold_with(&mut FieldAccessFolder {
             mark: self.mark,
+            method_mark: self.method_mark,
+            private_methods: &private_methods,
             statics: &statics,
             vars: vec![],
             class_name: &ident,
